c% ComputeEigenvalues: written by Antwan D. Clark. 
% This program computes the eigenvalues for 2-D the two-state stochastic fluid
% queue (SFQ) for both Markovian (lambda_mn != lambda_nm) and Poisson
% (lambda_mn = lambda_nm) cases. The eigenvalues are computed for each
% Fourier mode, which is part of the exact solutions for W^m and W^n. 
%
% For the Markovian Case, this program takes into account the stability of
% the solutions via estimating the range of Fourier modes where each mode
% of the solutions W^m and W^n are stable. From this, the same program can
% be used for a spectrum of stable eigenvalues. These eigenvalues can be
% incorporated to produce approximate solutions for W^m and W^n. 
%
% Input Parameters:
%  1.) lambda_mn -- System failure rate going from State m to State n. 
%  2.) lambda_nm -- System failure rate going from State n to State m.
%  3.) a_m -- cyber attack rate in State m
%  4.) a_n -- cyber attafck rate in State n.
%  5.) K -- the magnitude of the maximum Fourier mode number. The span of
%  Fourier modes ranges from -K to K.
%
% Output Parameters: 
% @TODO: Define Output Parameters as code develops. 


% Step 1: Defining input parameters. (Right now, just test for Poisson
% Case)
lambda_mn = 3.75; 
lambda_nm = 3.5; 
K = 12; 
a_m = 2; 
a_n = 3; 

%Step 2: Allocating the eigenvalue spectrum
kk = -K:K;

%Prestep: initializing vectors
c1 = zeros(1,length(kk));
c2 = zeros(1,length(kk));
phi = zeros(1, length(kk)); % This is for the Poisson case.

% Initializing for the Markovian case as well as preparing for Theorem 1.
RSquared = zeros(1, length(kk));
phi_k = zeros(1, length(kk));
varpi_k = zeros(1, length(kk));

%Allocating for recording unstable eigenvalues and modes
c1_unstable = zeros(1,length(kk));  
kk_unstable = zeros(1,length(kk));
count = 1; %initializing counter for recording unstable modes.

% This is to keep track of the associate location within the vector
% corresponding to the unstable modes. 
unstableIndex = zeros(1,length(kk)); 

% Step 2: Compute eigenvalues for each mode. If lambda_mn = lambda_nm, then
% all of the eigenvalues are stable since the real part is always
% non-positive (i.e. <= 0). Otherwise, we would need to check the
% conditions based on Theorem 1.

if lambda_mn ==lambda_nm %Poisson case.
    lambda = lambda_mn; 
    for j = 1:length(kk)
        phi(j) = sqrt(lambda^2 - kk(j)^2*((a_n - a_m)/2)^2);
        c1(j) = -lambda + 1i*kk(j)*((a_m + a_n)/2) + phi(j); 
        c2(j) = -lambda + 1i*kk(j)*((a_m + a_n)/2) - phi(j);
    end % end for loop
else %lambda_mn != lambda_nm (Markov case). 
    for j =1:length(kk)
    % Step 2a: Computing R^2, phi_k, and varpi_k
        RSquared(j) = ((lambda_mn + lambda_nm)^2 -(kk(j))^2*(a_n - a_m)^2)^2 + 4*(kk(j))^2*(a_n - a_m)^2*(lambda_mn - lambda_nm)^2;
        R(j) = sqrt(RSquared(j));
        phi_k(j) = sqrt((R(j) + ((lambda_mn + lambda_nm)^2 -(kk(j))^2*(a_n - a_m)^2))/2);
        varpi_k(j) = sqrt((R(j) - ((lambda_mn + lambda_nm)^2 -(kk(j))^2*(a_n - a_m)^2))/2);
    
        %Step 2b: Compute eigenvalue spectrum
        c1(j) = -((lambda_mn + lambda_nm) + 1i*kk(j)*(a_m + a_n))/2 + (phi_k(j) + varpi_k(j))/2;
        c2(j) = -((lambda_mn + lambda_nm) + 1i*kk(j)*(a_m + a_n))/2 - (phi_k(j) + varpi_k(j))/2;
    
        %Step 3: Apply Theorem 1. The goal is to perform a check to determine
        %whether the eigenvalue falls within the instability region defined in
        %Theorem 1. If it does, then the eigenvalue, Fourier mode, and location
        %within each vector is recorded. 
        dist = (real(c1(j) + ((lambda_mn + lambda_nm)/2)^2 + (imag(c1(j)) +kk(j)*(a_m + a_n)/2)^2));
        if (real(c1(j)) > 0 && dist <= RSquared(j)/4)
            c1_unstable(count) = c1(j); % Recording unstable eigenvalue
            kk_unstable(count) = kk(j); % Recording corresponding unstable mode.
            unstableIndex(count) = j; % Recording corresponding index.
            count =count+ 1; 
        end 
    end
end

% Next, gathering the growth rate data to plot corresponding to each
% Fourier mode.





