% ComputeEigenvalues: written by Antwan D. Clark. 
% This program computes the eigenvalues for 3-D the two-state stochastic fluid
% queue (SFQ) for both Markovian (lambda_mn != lambda_nm) and Poisson
% (lambda_mn = lambda_nm) cases. The eigenvalues are computed for each
% Fourier mode, which is part of the exact solutions for W^m and W^n. 
%
% For the Markovian Case, this program takes into account the stability of
% the solutions via estimating the range of Fourier modes where each mode
% of the solutions W^m and W^n are stable. From this, the same program can
% be used for a spectrum of stable eigenvalues. These eigenvalues can be
% incorporated to produce approximate solutions for W^m and W^n. 
%
% Input Parameters:
%  1.) lambda_mn -- System failure rate going from State m to State n. 
%  2.) lambda_nm -- System failure rate going from State m to State n.
%  3.) a_m -- cyber attack rate in State m.
%  4.) a_n -- cyber attafck rate in State n.
%  5.) r_m -- cyber infected rate in State m.
%  6.) r_n -- cyber infected rate in state n.
%  5.) K -- the magnitude of the maximum Fourier mode number. The span of
%  Fourier modes ranges from -K to K.
%  6.) L -- the magnitude of the maximum Fourier mode number. The span of
%  Fourier modes ranges from -L to L.
%
% Output Parameters: 
% @TODO: Define Output Parameters as code develops. 


% Step 1: Defining input parameters. (Right now, just test for Poisson
% Case)
lambda_mn = 3.5; 
lambda_nm = 3.5; 
K = 12; 
L = 12;
a_m = 2; 
a_n = 3; 
r_m = 2;
r_n = 1;

%Step 2: Allocating the eigenvalue spectrum
kk = -K:K;
ll = -L:L;

%Prestep: initializing vectors
c1 = zeros(length(kk),length(ll));
c2 = zeros(length(kk),length(ll));
phi =zeros(length(kk),length(ll)); % This is for the Poisson case.

% Initializing for the Markovian case as well as preparing for Theorem 1.
RSquared = zeros(length(kk),length(ll));
phi_kl = zeros(length(kk),length(ll));
varpi_kl = zeros(length(kk),length(ll));

%Allocating for recording unstable eigenvalues and modes
c1_unstable = zeros(length(kk),length(ll));  
kk_unstable = zeros(1,length(kk));
ll_unstable = zeros(1,length(kk));
countx = 1; %initializing counter for recording unstable modes.
county = 1;

% This is to keep track of the associate location within the vector
% corresponding to the unstable modes. 
unstableIndex = zeros(length(kk),length(ll)); 

% Step 2: Compute eigenvalues for each mode. If lambda_mn = lambda_nm, then
% all of the eigenvalues are stable since the real part is always
% non-positive (i.e. <= 0). Otherwise, we would need to check the
% conditions based on Theorem 1.

if lambda_mn ==lambda_nm %Poisson case.
    lambda = lambda_mn; 
    for j = 1:length(kk)
        for s=1:length(ll)
        phi(j,s) = sqrt(lambda^2 - (kk(j)*((a_n - a_m)/2)+ll(s)*((r_n-r_m)/2)^2));
        c1(j,s) = -lambda + 1i*(kk(j)*((a_m + a_n)/2)+ll(s)*((r_m+r_n)/2)) + phi(j,s); 
        c2(j,s) = -lambda + 1i*(kk(j)*((a_m + a_n)/2)+ll(s)*((r_m+r_n)/2)) - phi(j,s);
        end % end for l loop
    end % end for k loop
else %lambda_mn != lambda_nm (Markov case). 
    for j =1:length(kk)
        for s =1:length(ll)
    % Step 2a: Computing R^2, phi_k, and varpi_k
        RSquared(j,s) = ((lambda_mn + lambda_nm)^2 -(kk(j))^2*(a_n - a_m)^2-(ll(s)^2*(r_n-r_m)^2))^2 + 4*((kk(j))*(a_n - a_m)*(lambda_mn - lambda_nm)^2+(ll(s)*(r_n-r_m))*(lambda_mn - lambda_nm))^2;
        R(j,s) = sqrt(RSquared(j,s));
        phi_kl(j,s) = sqrt((R(j,s) + ((lambda_mn + lambda_nm)^2 -(kk(j))^2*(a_n - a_m)^2-(ll(s))^2*(r_n-r_m)^2)/2));
        varpi_kl(j,s) = sqrt((R(j,s) - ((lambda_mn + lambda_nm)^2 -(kk(j))^2*(a_n - a_m)^2-(ll(s))^2*(r_n-r_m)^2)/2));
    
        %Step 2b: Compute eigenvalue spectrum
        c1(j,s) = -((lambda_mn + lambda_nm) + 1i*kk(j)*(a_m + a_n)+1i*ll(s)*(r_m+r_n))/2 + (phi_kl(j,s) + varpi_kl(j,s))/2;
        c2(j,s) = -((lambda_mn + lambda_nm) + 1i*kk(j)*(a_m + a_n)+1i*ll(s)*(r_m+r_n))/2 - (phi_kl(j,s) + varpi_kl(j,s))/2;
    
        %Step 3: Apply Theorem 1. The goal is to perform a check to determine
        %whether the eigenvalue falls within the instability region defined in
        %Theorem 1. If it does, then the eigenvalue, Fourier mode, and location
        %within each vector is recorded. 
        dist = (real(c1(j,s) + ((lambda_mn + lambda_nm)/2)^2 + (imag(c1(j,s)) +kk(j)*(a_m + a_n)/2 +ll(s)*(r_m+r_n)/2)^2));
            if (real(c1(j,s)) > 0 && dist <= RSquared(j,s)/4)
                c1_unstable(countx,county) = c1(j,s); % Recording unstable eigenvalue
                kk_unstable(countx) = kk(j); % Recording corresponding unstable mode.
                ll_unstable(county) = ll(s); 
            end
        end 
    end
end

% Next, gathering the growth rate data to plot corresponding to each
% Fourier mode.





